use axum::{extract::Query, routing::get, Router};
use cairo::{Context, Format, ImageSurface};
use serde::{de, Deserialize};

use crate::{color::Color, polygon, SharedState};

fn default_as_false() -> bool {
    false
}

fn deserialize_bool<'de, D>(deserializer: D) -> Result<bool, D::Error>
where
    D: de::Deserializer<'de>,
{
    let s: &str = de::Deserialize::deserialize(deserializer)?;

    match s {
        "true" | "on" => Ok(true),
        "false" | "off" => Ok(false),
        _ => Err(de::Error::unknown_variant(
            s,
            &["true", "on", "false", "off"],
        )),
    }
}

#[derive(Deserialize)]
struct ImageProperties {
    #[serde(default = "default_as_false")]
    #[serde(deserialize_with = "deserialize_bool")]
    dark_mode: bool,
    background_color: Option<Color>,
}

async fn handler(Query(properties): Query<ImageProperties>) -> impl axum::response::IntoResponse {
    let surface = ImageSurface::create(Format::ARgb32, 400, 400).unwrap();
    let context = Context::new(&surface).unwrap();

    if let Some(c) = properties.background_color {
        context.set_source_rgb(c.r as f64 / 255.0, c.g as f64 / 255.0, c.b as f64 / 255.0);
    } else {
        context.set_source_rgba(0.0, 0.0, 0.0, 0.0);
    }

    context.paint().unwrap();

    polygon::draw_polygon_of_segmented_polygons(
        (200.0, 200.0),
        200.0,
        6,
        &context,
        properties.dark_mode,
    )
    .unwrap();

    let mut data: Vec<u8> = Vec::new();
    surface.write_to_png(&mut data).unwrap();

    (
        axum::response::AppendHeaders([(axum::http::header::CONTENT_TYPE, "image/png")]),
        data,
    )
}

pub fn routes() -> Router<SharedState> {
    Router::new().route("/", get(handler))
}
